/*
 * lcd1602
 */

#include "lcd1602.h"
#include "lm7000.h"

code unsigned char * const text_FM  = "FM";
code unsigned char * const text_AM  = "AM";
code unsigned char * const text_MHz = "MHz";
code unsigned char * const text_KHz = "KHz";

u8 text_freq_disp[7];

//lcd1602初始化
void lcd1602_init(void)
{
	//初始化流程
	//	1. Display clear
	//	2. Function set:
	//		DL = 1; 8-bit interface data
	//		N = 1; 2-line display
	//		F = 0; 5x8 dot character font
	//	3. Display on/off control:
	//		D = 1; Display on
	//		C = 0; Cursor off
	//		B = 0; Blinking off
	//	4. Entry mode set:
	//		I/D = 1; Increment by 1
	//		S = 0; No shift
	//命令总汇
	//                         7    6    5    4    3     2     1     0
	//Clear display:           0    0    0    0    0     0     0     1   // 0x01
	//Return home:             0    0    0    0    0     0     1     x   // 0x02 or 0x03
	//Entry mode set:          0    0    0    0    0     1     [I/D] [S] // 0x04 + [I增/D减] [S偏移]
	//Display on/off control:  0    0    0    0    1     [D]   [C]   [B] // 0x08 + [D显示] [C光标] [B闪烁]
	//Cursor or display shift: 0    0    0    1    [S/C] [R/L] x     x   // 0x10 + [S字符移动/C光标移动] [R右/L左]
	//Function set:            0    0    1    [DL] [N]   [F]   x     x   // 0x20 + [DL 8bit/4bit] [N 2行/1行] [F=0 5x8-dots]
	//Set CGRAM address:       0    1    ACG  ACG  ACG   ACG   ACG   ACG // 0x40 + CGRAM address
	//Set DDRAM address：      1    ADD  ADD  ADD  ADD   ADD   ADD   ADD // 0x80 + DDRAM address
	Delay10ms();
	Delay10ms();
	Delay10ms();
	Delay10ms();
	Delay10ms(); //等待电源稳定
	//                            D/L        N          F
	lcd1602_write_command(0x20 + ((1)<<4) + ((1)<<3) + ((0)<<2)); //不能检查BUSY
	Delay10ms();
	lcd1602_write_command(0x20 + ((1)<<4) + ((1)<<3) + ((0)<<2)); //不能检查BUSY
	Delay10ms();
	lcd1602_write_command(0x20 + ((1)<<4) + ((1)<<3) + ((0)<<2)); //不能检查BUSY
	Delay10ms();
	//                            D          C          B
 	lcd1602_write_command(0x08 + ((1)<<2) + ((0)<<1) + ((0)<<0));
	Delay10ms();
	//                            I/D        S
	lcd1602_write_command(0x04 + ((1)<<1) + ((0)<<0));
	Delay10ms();
 	lcd1602_write_command(0x01);
	Delay10ms();
}

void lcd1602_wait_busy(void)
{
	u8 d;
	
	LCDE = 0;
	LCDRS = 0;
	LCDRW = 1;
	do {
		LCDE = 1;
		d = P0;
		LCDE = 0;
	} while (d & 0x80);
	LCDRW = 0;
}

void lcd1602_write_command(u8 cmd)
{
	LCDE = 0;
	LCDRS = 0;
	LCDRW = 0;
 	LCDE = 1;
 	LCDDATA = cmd;
 	LCDE = 0;
}

void lcd1602_write_data(u8 d)
{
	LCDE = 0;
	LCDRS = 1;
	LCDRW = 0;
 	LCDE = 1;
 	LCDDATA = d;
 	LCDE = 0;
	LCDRS = 0;
}

void lcd1602_disp_char(u8 ddram, u8 c)
{
	lcd1602_wait_busy();
 	lcd1602_write_command(0x80 + ddram);
	lcd1602_wait_busy();
 	lcd1602_write_data(c); 
}

void lcd1602_disp_string(u8 ddram, unsigned char *p, u8 length)
{
	u8 i;
	
 	for (i = 0; i < length; i++)
	{
		lcd1602_disp_char(ddram++, *(p+i));
	}
}

void lcd1602_disp_band_freq(void)
{
	if (band == FM)
	{
		text_freq_disp[0] = fm_rx_freq / 100000000 % 10 + '0';
		text_freq_disp[1] = fm_rx_freq / 10000000 % 10 + '0';
		text_freq_disp[2] = fm_rx_freq / 1000000 % 10 + '0';
		text_freq_disp[3] = '.';
		text_freq_disp[4] = fm_rx_freq / 100000 % 10 + '0';
		text_freq_disp[5] = fm_rx_freq / 10000 % 10 + '0';
		text_freq_disp[6] = fm_rx_freq / 1000 % 10 + '0';
		
		lcd1602_disp_string(LCD_LINE2_DDRAM+0, text_FM, 2);
		
		if (text_freq_disp[0] == '0')
		{
			//不显示最高位0
			lcd1602_disp_string(LCD_LINE2_DDRAM+3, text_freq_disp+1, 6);
			lcd1602_disp_string(LCD_LINE2_DDRAM+3+6, text_MHz, 3);
			lcd1602_disp_char(LCD_LINE2_DDRAM+3+6+3, ' ');
		}
		else
		{
			lcd1602_disp_string(LCD_LINE2_DDRAM+3, text_freq_disp, 7);
			lcd1602_disp_string(LCD_LINE2_DDRAM+3+7, text_MHz, 3);
		}
	}
	else
	{
		text_freq_disp[0] = am_rx_freq / 1000000 % 10 + '0';
		text_freq_disp[1] = am_rx_freq / 100000 % 10 + '0';
		text_freq_disp[2] = am_rx_freq / 10000 % 10 + '0';
		text_freq_disp[3] = am_rx_freq / 1000 % 10 + '0';
		
		lcd1602_disp_string(LCD_LINE2_DDRAM+0, text_AM, 2);
		
		if (text_freq_disp[0] == '0')
		{
			//不显示最高位0
			lcd1602_disp_string(LCD_LINE2_DDRAM+3, text_freq_disp+1, 3);
			lcd1602_disp_string(LCD_LINE2_DDRAM+3+3, text_KHz, 3);
			lcd1602_disp_string(LCD_LINE2_DDRAM+3+3+3, text_blank, 4);
		}
		else
		{
			lcd1602_disp_string(LCD_LINE2_DDRAM+3, text_freq_disp, 4);
			lcd1602_disp_string(LCD_LINE2_DDRAM+3+4, text_KHz, 3);
			lcd1602_disp_string(LCD_LINE2_DDRAM+3+4+3, text_blank, 3);
		}
	}
}
