
#include "lm7000.h"
#include "lcd1602.h"

volatile bdata lm7000_data0 = 0; // [D0..D7]
volatile bdata lm7000_data1 = 0; // [D8..D13] [T0] [T1]
volatile bdata lm7000_data2 = 0; // [B0] [B1] [B2] [TB] [R0] [R1] [R2] [S]

//[D0..D13] FM
//[D4..D13] AM
//Ƶϵ㷽FM VCO/Fref
//FM VCOFM RF + IF
//Fref[R0..R3]
//[T0..T1]ǲλΪ

//Band data  Time base data
//[B0..B2] = 000ʱ[R0..R2]Զ

//Reference frequency dataοƵ
// R0  R1  R2       Fref       BO1  BO2  BO3
// IF 10.7MHz
// 0   0   0        100K       1    1    0
// 0   0   1         50K       1    1    0
// 0   1   0         25K       1    1    0
// IF 450KHz
// 0   1   1          5K       0    0    1
// 1   0   0         10K       1    0    1
// 1   0   1          9K       1    0    1
// 1   1   0          1K       0    1    1
// 1   1   1          5K       0    0    1

// Divider select data Ƶѡ
// '1':FMIN, '0':AMIN

#define FM_FREF_100K  (100000UL)
#define FM_FREF_50K   (50000UL)
#define FM_FREF_25K   (25000UL)
#define AM_FREF_10K   (10000UL)
#define AM_FREF_9K    (9000UL)
#define AM_FREF_5K    (5000UL)
#define AM_FREF_1K    (1000UL)

u8 band;
u32 fm_rx_freq;
u32 fm_fref;
u32 am_rx_freq;
u32 am_fref;

static void lm7000_set_am_freq(void);
static void lm7000_set_fm_freq(void);
static void lm7000_write(void);

void lm7000_init(void)
{
	LM7000_CE = 0;
	LM7000_CLK = 0;
	LM7000_DATA = 0;
	
	band = FM;
	am_rx_freq = 999000;
	am_fref = AM_FREF_9K;
	fm_rx_freq = 104300000;
	fm_fref = FM_FREF_25K;
	
	lm7000_set_band();
}

void lm7000_set_band(void)
{
	//л
	//[B0..B2] [TB] лӲ˿
	//RX-V2090 BO1(PIN8)=0/BO2(PIN9)=1AMBO1(PIN8)=1/BO2(PIN9)=0FM
	//[R0..R2]      òοƵ
	//[S]           ÷Ƶѡ
	if (band == FM)
	{
		// S = 1;   //ƵѡFMIN
		// R2 = 0;  //010ѡ25KHzοƵʣ25KHz
		// R1 = 1;
		// R0 = 0;
		// TB = 0;  //ʱ
		// BO3 = 0; //δʹ
		// BO2 = 0; //FM
		// BO1 = 1; //AM
		// [S] [R2] [R1] [R0] [TB] [B2] [B1] [B0]
		lm7000_data2 = B10100001;
		fm_fref = FM_FREF_25K;
		lm7000_set_fm_freq();
	}
	else
	{
		// S = 0;   //ƵѡAMIN
		// R2 = 1;  //101ѡ9KHzοƵʣ9KHz
		// R1 = 0;
		// R0 = 1;
		// TB = 0;  //ʱ
		// BO3 = 0; //δʹ
		// BO2 = 1; //FM
		// BO1 = 0; //AM
		// [S] [R2] [R1] [R0] [TB] [B2] [B1] [B0]
		lm7000_data2 = B01010010;
		am_fref = AM_FREF_9K;
		lm7000_set_am_freq();
	}
	
	//ݵLM7000
	lm7000_write();
	
	//ʾǰƵʵ1602
	lcd1602_disp_band_freq();
}

void lm7000_set_freq(void)
{
	if (band == FM)
	{
		lm7000_set_fm_freq();
	}
	else
	{
		lm7000_set_am_freq();
	}
	
	//ݵLM7000
	lm7000_write();
	
	//ʾǰƵʵ1602
	lcd1602_disp_band_freq();
}

static void lm7000_set_am_freq(void)
{
	u32 f_vco;
	u32 div;
	
	//ĿƵʣAMƵϵ
	if (am_rx_freq <= AM_FREQ_MAX && am_rx_freq >= AM_FREQ_MIN)
	{
		f_vco = am_rx_freq + AM_FIF;
		div = f_vco / am_fref;
		div <<= 4;
		lm7000_data0 = (div >> 0) & 0xff; //D4..D7
		lm7000_data1 = (div >> 8) & 0xff; //D8..D13
	}
}

static void lm7000_set_fm_freq(void)
{
	u32 f_vco;
	u32 div;
	
	//ĿƵʣFMƵϵ
	if (fm_rx_freq <= FM_FREQ_MAX && fm_rx_freq >= FM_FREQ_MIN)
	{
		f_vco = fm_rx_freq + FM_FIF;
		div = f_vco / fm_fref;
		lm7000_data0 = (div >> 0) & 0xff; //D0..D7
		lm7000_data1 = (div >> 8) & 0xff; //D8..D13
	}
}

//LM7000ݴ亯
static void lm7000_write(void)
{
	u8 i;
	u8 d;
	
	LM7000_CE = 1;
	
	d = lm7000_data0;
	
	for (i = 8; i > 0; i--)
	{
		if (d & 0x01)
		{
			LM7000_DATA = 1;
		}
		else
		{
			LM7000_DATA = 0;
		}
		
		LM7000_CLK = 1;
		LM7000_DELAY();
		LM7000_CLK = 0;
		LM7000_DELAY();
		
		d >>= 1;
	}
	
	d = lm7000_data1;
	
	for (i = 8; i > 0; i--)
	{
		if (d & 0x01)
		{
			LM7000_DATA = 1;
		}
		else
		{
			LM7000_DATA = 0;
		}
		
		LM7000_CLK = 1;
		LM7000_DELAY();
		LM7000_CLK = 0;
		LM7000_DELAY();
		
		d >>= 1;
	}
	
	d = lm7000_data2;
	
	for (i = 8; i > 0; i--)
	{
		if (d & 0x01)
		{
			LM7000_DATA = 1;
		}
		else
		{
			LM7000_DATA = 0;
		}
		
		LM7000_CLK = 1;
		LM7000_DELAY();
		LM7000_CLK = 0;
		LM7000_DELAY();
		
		d >>= 1;
	}
	
	LM7000_CE = 0;
	LM7000_CLK = 0;
	LM7000_DATA = 0;
}


