
#include "uart1.h"

volatile bit tx_busy = 0;
volatile bit rx_busy = 0;
u8 rx_buf = 0;

void uart_init(void)
{
    SCON = 0x50;                    //UART:N-8-1
    TMOD = 0x20;                    //Set Timer1 as 8-bit auto reload mode
    TH1 = TL1 = -(FOSC/12/32/BAUD); //Set auto-reload vaule
    TR1 = 1;                        //Timer1 start run
    ES = 1;                         //Enable UART interrupt
}

/*----------------------------
UART interrupt service routine
----------------------------*/
void Uart_Isr() interrupt 4
{
    if (RI)
    {
        RI = 0;             //Clear receive interrupt flag
        rx_buf = SBUF;          //P0 show UART data
				rx_busy = 1;
    }
		
    if (TI)
    {
        TI = 0;             //Clear transmit interrupt flag
        tx_busy = 0;           //Clear transmit busy flag
    }
}

/*----------------------------
Send a byte data to UART
Input: dat (data to be sent)
Output:None
----------------------------*/
void SendData(u8 d)
{
    while (tx_busy);           //Wait for the completion of the previous data is sent
    tx_busy = 1;
    SBUF = d;             //Send data to UART buffer
}

/*----------------------------
Send a string to UART
Input: s (address of string)
Output:None
----------------------------*/
void SendString(unsigned char *s)
{
    while (*s)              //Check the end of the string
    {
        SendData(*s++);     //Send current char and increment string ptr
    }
}
